@@ -8,11 +8,11 @@ from sql import Null
 from trytond.cache import Cache
 from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, fields, sequence_ordered, tree, DeactivableMixin)
-from trytond.transaction import Transaction
+    DeactivableMixin, ModelSQL, ModelView, fields, sequence_ordered, tree)
 from trytond.pool import Pool
-from trytond.pyson import Eval, If, Bool, PYSONEncoder
-from trytond.tools import reduce_ids, grouped_slice
+from trytond.pyson import Bool, Eval, If, PYSONEncoder
+from trytond.tools import grouped_slice, reduce_ids
+from trytond.transaction import Transaction
 
 from .exceptions import WorkProgressValidationError
 
@@ -132,12 +132,16 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
     party = fields.Many2One('party.party', 'Party',
         states={
             'invisible': Eval('type') != 'project',
-            }, depends=['type'])
+            },
+        context={
+            'company': Eval('company', -1),
+            },
+        depends={'company'})
     party_address = fields.Many2One('party.address', 'Contact Address',
         domain=[('party', '=', Eval('party'))],
         states={
             'invisible': Eval('type') != 'project',
-            }, depends=['party', 'type'])
+            })
     timesheet_works = fields.One2Many(
         'timesheet.work', 'origin', 'Timesheet Works', readonly=True, size=1)
     timesheet_available = fields.Function(
@@ -146,14 +150,12 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
     timesheet_start_date = fields.Function(fields.Date('Timesheet Start',
             states={
                 'invisible': ~Eval('timesheet_available'),
-                },
-            depends=['timesheet_available']),
+                }),
         'get_timesheet_date', setter='set_timesheet_date')
     timesheet_end_date = fields.Function(fields.Date('Timesheet End',
             states={
                 'invisible': ~Eval('timesheet_available'),
-                },
-            depends=['timesheet_available']),
+                }),
         'get_timesheet_date', setter='set_timesheet_date')
     timesheet_duration = fields.Function(fields.TimeDelta('Duration',
             'company_work_time',
@@ -183,23 +185,19 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
                 }),
         'get_total')
     comment = fields.Text('Comment')
-    parent = fields.Many2One('project.work', 'Parent',
-        left='left', right='right', ondelete='RESTRICT',
+    parent = fields.Many2One(
+        'project.work', 'Parent', path='path', ondelete='RESTRICT',
         domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-    left = fields.Integer('Left', required=True, select=True)
-    right = fields.Integer('Right', required=True, select=True)
+            ])
+    path = fields.Char("Path", select=True)
     children = fields.One2Many('project.work', 'parent', 'Children',
         domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
     status = fields.Many2One(
         'project.work.status', "Status", required=True, select=True,
-        domain=[If(Bool(Eval('type')), ('types', 'in', Eval('type')), ())],
-        depends=['type'])
+        domain=[If(Bool(Eval('type')), ('types', 'in', Eval('type')), ())])
 
     @staticmethod
     def default_type():
@@ -215,18 +213,12 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
         WorkStatus = pool.get('project.work.status')
         return WorkStatus.get_default_status(cls.default_type())
 
-    @classmethod
-    def default_left(cls):
-        return 0
-
-    @classmethod
-    def default_right(cls):
-        return 0
-
     @classmethod
     def __register__(cls, module_name):
         TimesheetWork = Pool().get('timesheet.work')
-        cursor = Transaction().connection.cursor()
+        transaction = Transaction()
+        cursor = transaction.connection.cursor()
+        update = transaction.connection.cursor()
         table_project_work = cls.__table_handler__(module_name)
         project = cls.__table__()
         timesheet = TimesheetWork.__table__()
@@ -245,9 +237,9 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
         if table_project_work.column_exist('effort'):
             cursor.execute(*project.select(project.id, project.effort,
                     where=project.effort != Null))
-            for id_, effort in cursor.fetchall():
+            for id_, effort in cursor:
                 duration = datetime.timedelta(hours=effort)
-                cursor.execute(*project.update(
+                update.execute(*project.update(
                         [project.effort_duration],
                         [duration],
                         where=project.id == id_))
@@ -263,8 +255,8 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
                     condition=timesheet.parent == second_project.work
                     ).select(project.id, second_project.id)
             cursor.execute(*query)
-            for id_, parent in cursor.fetchall():
-                cursor.execute(*project.update(
+            for id_, parent in cursor:
+                update.execute(*project.update(
                         [project.parent],
                         [parent],
                         where=project.id == id_))
@@ -273,8 +265,8 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
             cursor.execute(*project.join(timesheet,
                     condition=project.work == timesheet.id
                     ).select(project.id, timesheet.company))
-            for id_, company in cursor.fetchall():
-                cursor.execute(*project.update(
+            for id_, company in cursor:
+                update.execute(*project.update(
                         [project.company],
                         [company],
                         where=project.id == id_))
@@ -283,8 +275,8 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
             cursor.execute(*project.join(timesheet,
                     condition=project.work == timesheet.id
                     ).select(project.id, timesheet.name))
-            for id_, name in cursor.fetchall():
-                cursor.execute(*project.update(
+            for id_, name in cursor:
+                update.execute(*project.update(
                         [project.name],
                         [name],
                         where=project.id == id_))
@@ -292,7 +284,6 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
         # Migration from 4.0: remove work
         if work_exist:
             table_project_work.drop_constraint('work_uniq')
-            update = Transaction().connection.cursor()
             cursor.execute(*project.select(project.id, project.work,
                     where=project.work != Null))
             for project_id, work_id in cursor:
@@ -305,6 +296,10 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
         # Migration from 5.4: replace state by status
         table_project_work.not_null_action('state', action='remove')
 
+        # Migration from 6.0: remove left and right
+        table_project_work.drop_column('left')
+        table_project_work.drop_column('right')
+
     @fields.depends('type', 'status')
     def on_change_type(self):
         pool = Pool()
@@ -459,7 +454,7 @@ class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
             where = reduce_ids(table.id, sub_ids)
             cursor.execute(*table.select(table.id, table.parent,
                     where=where))
-            parents.update(cursor.fetchall())
+            parents.update(cursor)
 
         if 'total_progress' in names and 'total_effort' not in names:
             names = list(names)
